#version 130


/* DRAWBUFFERS:3 */
/* SHADOWRES:2048 */ 
/* SHADOWHPL:58 */   

//#define FAR_DISTANCE_SHADOW

uniform sampler2D gcolor;
uniform sampler2D gnormal;
uniform sampler2D gaux1;   // specular map
uniform sampler2D gaux2;   // weather
uniform sampler2D colortex1;   // ambient light
uniform sampler2D shadow;
uniform sampler2D watershadow;
uniform sampler2D depthtex0;
uniform sampler2D depthtex1;
uniform sampler2D shadowcolor;

in vec4 texcoord;
in vec3 lightVector;
//varying vec3 specMultiplier;

uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelViewInverse;
uniform mat4 shadowProjection;
uniform mat4 shadowModelView;

uniform int fogMode;
uniform int worldTime;
uniform vec3 sunPosition;
uniform vec3 moonPosition;
uniform float sunAngle;
uniform float rainStrength;

in vec3 heldLightSpecMultiplier;
in float heldLightMagnitude;

const int FOGMODE_LINEAR 	= 9729;
const int FOGMODE_EXP 		= 2048;
const float PI 				= 3.1415927;
const float sunPathRotation = -30.0;
vec4 fcolor   				= texture2D(gcolor, texcoord.st);
vec4 fnormal  				= texture2D(gnormal, texcoord.st);
vec4 faux1    				= texture2D(gaux1, texcoord.st);
vec4 faux2    				= texture2D(gaux2, texcoord.st);
vec4 fambient 				= texture2D(colortex1, texcoord.st);
float depth0  				= texture2D(depthtex0, texcoord.st).x;
float depth1  				= texture2D(depthtex1, texcoord.st).x;

bool isNight;

void calcShadowShading(in vec3 prjpos, out vec3 nfpos, out vec3 shading)
{
	vec4 fragposition = gbufferProjectionInverse * vec4(prjpos.x * 2.0 - 1.0, prjpos.y * 2.0 - 1.0, prjpos.z * 2.0 - 1.0, 1.0);
	fragposition /= fragposition.w;
	nfpos = normalize(fragposition.xyz);

	float distance = sqrt(fragposition.x * fragposition.x + fragposition.y * fragposition.y + fragposition.z * fragposition.z);
	
	shading = vec3(1.0);
	
	vec4 shadowColorSample = vec4(1.0);
	
	float hpl;
#ifdef FAR_DISTANCE_SHADOW
	hpl = 256 * 256;
#else
	hpl = 64 * 64;
#endif
	
	if (distance < hpl) {
		// shadows
		vec4 worldposition = gbufferModelViewInverse * fragposition;

		float xzDistanceSquared = worldposition.x * worldposition.x + worldposition.z * worldposition.z;
		float yDistanceSquared  = worldposition.y * worldposition.y;
		
		if (yDistanceSquared < (hpl * 0.75)) {
			worldposition = shadowModelView * worldposition;
			float comparedepth = -worldposition.z;
			worldposition = shadowProjection * worldposition;
			worldposition /= worldposition.w;
			
			worldposition.st = worldposition.st / 2 + 0.5;
			
			if (comparedepth > 0.0 && worldposition.s < 1.0 && worldposition.s > 0.0 && worldposition.t < 1.0 && worldposition.t > 0.0){
				float shadowMult = clamp(1.0 - xzDistanceSquared / hpl, 0.0, 1.0) * clamp(1.0 - yDistanceSquared / hpl, 0.0, 1.0) + 0.3;
				
				vec2 shadowPos = worldposition.st;
				float shadowSample = texture2D(shadow, shadowPos).r;
				float waterShadowSample = texture2D(watershadow, shadowPos).r;
				
				float shadowDepth = 0.05 + shadowSample * (256.0 - 0.05);
				float waterShadowDepth = 0.05 + waterShadowSample * (256.0 - 0.05);
				
				float shading1 = 1.0 - shadowMult * (clamp(comparedepth - shadowDepth      - 0.05, 0.0, 1.0)) * clamp(1.0 - rainStrength,0.1,1.0);
				float shading2 = 1.0 - shadowMult * (clamp(comparedepth - waterShadowDepth - 0.05, 0.0, 1.0)) * clamp(1.0 - rainStrength,0.1,1.0);
				
				vec4 shadowColorSample = texture2D(shadowcolor, shadowPos);
				shading = shadowColorSample.rgb*(shading1 - shading2) + shading2;
			}
		}
	}
}

void main() {
	float depth = depth1;
	vec3 normal = fnormal.xyz * 2.0 - 1.0;
	vec3 dcolor = fcolor.rgb;
	vec3 ambient = fambient.rgb;
	vec4 comp;
	
	if (normal==vec3(0.0) || normal==vec3(-1.0))
	{
		comp = vec4(dcolor * ambient,1.0);
	}
	else
	{
		vec3 tsnfpos;
		vec3 tsshading;
		calcShadowShading(vec3(texcoord.st,depth0), tsnfpos, tsshading);

		vec3 specularColor = faux1.rgb;
		float sunmoonLevel = sin(sunAngle*(2*PI));
		isNight = (sunmoonLevel < 0);
		vec3 sunmoonColor;
		if (isNight)
		{
			sunmoonLevel *= (-0.5);
			float blue;
			if (sunmoonLevel < 0.1) {
				blue = sunmoonLevel / 2 + 0.5;
			} else if(sunmoonLevel < 0.15) {
				blue = sunmoonLevel / 2 + 0.47;
			} else if(sunmoonLevel < 0.2) {
				blue = sunmoonLevel / 2 + 0.42;
			} else if(sunmoonLevel < 0.25) {
				blue = sunmoonLevel / 2 + 0.4;
			} else if(sunmoonLevel < 0.3) {
				blue = sunmoonLevel / 2 + 0.37;
			} else if(sunmoonLevel < 0.35) {
				blue = sunmoonLevel / 2 + 0.32;
			} else if(sunmoonLevel < 0.4) {
				blue = sunmoonLevel / 2 + 0.3;
			} else if(sunmoonLevel < 0.45) {
				blue = sunmoonLevel / 2 + 0.27;
			} else {
				blue = sunmoonLevel / 2 + 0.25;
			}
			sunmoonColor = vec3(sunmoonLevel / 2 + 0.05, sunmoonLevel / 2 + 0.12, blue);
		} else {
			float red; float green;
			if (sunmoonLevel < 0.1) {
				red = sunmoonLevel / 2 + 0.66;
				green = sunmoonLevel / 2 + 0.3;
			} else if(sunmoonLevel < 0.15) {
				red = sunmoonLevel / 2 + 0.57;
				green = sunmoonLevel / 2 + 0.31;
			} else if(sunmoonLevel < 0.2) {
				red = sunmoonLevel / 2 + 0.5;
				green = sunmoonLevel / 2 + 0.28;
			} else if(sunmoonLevel < 0.25) {
				red = sunmoonLevel / 2 + 0.43;
				green = sunmoonLevel / 2 + 0.272;
			} else if(sunmoonLevel < 0.3) {
				red = sunmoonLevel / 2 + 0.4;
				green = sunmoonLevel / 2 + 0.25;
			} else if(sunmoonLevel < 0.35) {
				red = sunmoonLevel / 2 + 0.36;
				green = sunmoonLevel / 2 + 0.22;
			} else if(sunmoonLevel < 0.4) {
				red = sunmoonLevel / 2 + 0.31;
				green = sunmoonLevel / 2 + 0.2;
			} else if(sunmoonLevel < 0.45) {
				red = sunmoonLevel / 2 + 0.298;
				green = sunmoonLevel / 2 + 0.19;
			} else {
				red = sunmoonLevel / 2 + 0.28;
				green = sunmoonLevel / 2 + 0.16;
			}
			sunmoonColor = vec3(red, green,	sunmoonLevel / 2 + 0.1); 
		}
		
		float s;
		normal = normalize(normal);
		s = max(dot(reflect(tsnfpos, normal), lightVector), 0.0);
		comp = vec4(min(dcolor * (ambient + max(dot(normal, lightVector),0.0) * sunmoonColor * tsshading) + specularColor * s * s * s * sunmoonColor * tsshading, 1.0), 1.0);
			
		if (depth1 > depth0) {
			vec3 uwnfpos; vec3 uwshading;
			calcShadowShading(vec3(texcoord.st,depth1), uwnfpos, uwshading);
			comp = comp * 0.5 + vec4(min(dcolor * (ambient + ambient * sunmoonColor * uwshading), 1.0), 1.0) * 0.5;
		}
	}
	
	if (rainStrength > 0) {
		comp.rgb = mix(comp.rgb, gl_Fog.color.rgb, (1.0 + (rainStrength * 0.5))-fambient.a);
	} else {
		comp.rgb = mix(comp.rgb, gl_Fog.color.rgb, 1.0-fambient.a);
	}
	
	// rain
	//comp.rgb = mix(comp.rgb, faux2.rgb, faux2.a);
	gl_FragData[0] = comp;
}
